Attribute VB_Name = "DDraw"
'**************************************************************
'
' THIS WORK, INCLUDING THE SOURCE CODE, DOCUMENTATION
' AND RELATED MEDIA AND DATA, IS PLACED INTO THE PUBLIC DOMAIN.
'
' THE ORIGINAL AUTHOR IS RYAN CLARK.
'
' THIS SOFTWARE IS PROVIDED AS-IS WITHOUT WARRANTY
' OF ANY KIND, NOT EVEN THE IMPLIED WARRANTY OF
' MERCHANTABILITY. THE AUTHOR OF THIS SOFTWARE,
' ASSUMES _NO_ RESPONSIBILITY FOR ANY CONSEQUENCE
' RESULTING FROM THE USE, MODIFICATION, OR
' REDISTRIBUTION OF THIS SOFTWARE.
'
'**************************************************************
'
' This file was downloaded from The Game Programming Wiki.
' Come and visit us at http://gpwiki.org
'
'**************************************************************
'
'IMPORTANT TO NOTE:
'
'When using resource files and the CreateSurfaceFromResource
'command in DirectX 7.0, you will not be able to run your program
'by simply pressing F5 or selecting the run menu item, you will
'get an error. dX7 will only recognize your resource file if you
'compile your program and RUN THE COMPILED VERSION.
'
'**************************************************************

'Major DX Objects
Dim dx As New DirectX7
Dim dd As DirectDraw7

Dim Primary As DirectDrawSurface7       'Primary surface
Dim BackBuffer As DirectDrawSurface7    'Backbuffer surface
Dim ddsdPrimary As DDSURFACEDESC2       'Primary surface description
Dim ddsdBackBuffer As DDSURFACEDESC2    'Backbuffer surface description

Dim Ship(39) As DirectDrawSurface7      'Array of surfaces that will contain our ship
Global Running As Boolean               'Determines if the program should still be running
Global Facing As Integer                'Determines the direction the ship is currently facing
Global Zoom As Single                   'How far are we zoomed?
Global ColourFill As Integer            'Variable that determines the colour of the background

Const SpriteWidth = 60                  'Width of the sprite we're displaying
Const SpriteHeight = 60                 'Height of the sprite we're displaying
    
Public Sub Initialize()

    'This routine initializes the display mode, and the primary/backbuffer complex
    
    'Handles errors
    On Local Error GoTo ErrOut
    
    'Creates the directdraw object
    Set dd = dx.DirectDrawCreate("")
       
    'Set the cooperative level and displaymode...
    Call dd.SetCooperativeLevel(FrmDirectX.hWnd, DDSCL_FULLSCREEN Or DDSCL_EXCLUSIVE Or DDSCL_ALLOWREBOOT)
    Call dd.SetDisplayMode(640, 480, 16, 0, DDSDM_DEFAULT)
    
    'Create the primary complex surface with one backbuffer
    ddsdPrimary.lFlags = DDSD_CAPS Or DDSD_BACKBUFFERCOUNT
    ddsdPrimary.ddsCaps.lCaps = DDSCAPS_PRIMARYSURFACE Or DDSCAPS_FLIP Or DDSCAPS_COMPLEX
    ddsdPrimary.lBackBufferCount = 1
    Set Primary = dd.CreateSurface(ddsdPrimary)
    
    'Get the backbuffer from the primary surface
    Dim caps As DDSCAPS2
    caps.lCaps = DDSCAPS_BACKBUFFER
    Set BackBuffer = Primary.GetAttachedSurface(caps)
    
    'Set the colour (for text output) of the backbuffer
    BackBuffer.SetForeColor vbWhite
    
    'Initialize the global variables
    Facing = 0
    Zoom = 1
    ColourFill = 0
    Running = True
    
    'Load the sprites
    LoadSurfaces
         
    'Clears the buffer
    ClearBuffer
    
    Exit Sub
    
ErrOut:
    Running = False             'If there's an error, exit the program
    
End Sub

Private Sub LoadSurfaces()

Dim i As Integer

    'This routine loads all of the surfaces we're going to be using
    For i = 0 To UBound(Ship)
        LoadSprite Ship(i), "vegan" & i, SpriteWidth, SpriteHeight, vbBlack
    Next

End Sub

Public Sub ClearBuffer()

Dim DestRect As RECT

    'This routine clears the backbuffer and displays the text
    With DestRect           'This rectangle must be defined the same as the screen (which we set as 640x480)
        .Bottom = 480
        .Left = 0
        .Right = 640
        .Top = 0
    End With
    
    'Fill the entire backbuffer with the colour dictated by "ColourFill"
    BackBuffer.BltColorFill DestRect, ColourFill
    
    'Draw the text on the backbuffer the describe the keys
    BackBuffer.DrawText 100, 100, "Press ESC to exit", False
    BackBuffer.DrawText 100, 120, "Left or Right arrows to animate", False
    BackBuffer.DrawText 100, 140, "'O' button zooms out, 'I' zooms back in", False
    BackBuffer.DrawText 100, 160, "and Up or Down arrows to change backdrop.", False

End Sub

Public Sub LoadSprite(ByRef Sprite As DirectDrawSurface7, File As String, bWidth As Integer, bHeight As Integer, ColourKey As Integer)

Dim CKey As DDCOLORKEY
Dim ddsdNewSprite As DDSURFACEDESC2
    
    'This routine loads sprites in the FObject file and sets their colour keys
    ddsdNewSprite.lFlags = DDSD_CAPS Or DDSD_WIDTH Or DDSD_HEIGHT           'Set the surface description to include the Capabilities, Width and Height
    ddsdNewSprite.ddsCaps.lCaps = DDSCAPS_OFFSCREENPLAIN                    'Set the surface's capabilities to be an offscreen surface
    ddsdNewSprite.lWidth = bWidth                                           'Set the width of the surface
    ddsdNewSprite.lHeight = bHeight                                         'Set the height of the surface
    Set Sprite = dd.CreateSurfaceFromResource("", File, ddsdNewSprite)      'Load the bitmap from the resource file into the surface using the surface description
    CKey.low = ColourKey                                                    'Set the low value of the colour key
    CKey.high = ColourKey                                                   'and the high value (in this case they're the same because we're not using a range)
    Sprite.SetColorKey DDCKEY_SRCBLT, CKey                                  'Set the sprites colourkey using the key just created

End Sub

Public Sub DisplaySprite()

Dim SrcRect As RECT
Dim DestRect As RECT

    'Set up the source rectangle
    With SrcRect
        .Bottom = SpriteHeight
        .Left = 0
        .Right = SpriteWidth
        .Top = 0
    End With
    
    'Set up the destination rectangle (taking zoom into account)
    With DestRect
        .Bottom = SpriteHeight * Zoom
        .Left = 0
        .Right = SpriteWidth * Zoom
        .Top = 0
    End With
    
    'Blit the surface on to the backbuffer at the specified location, using the colour key of the source
    BackBuffer.Blt DestRect, Ship(Facing), SrcRect, DDBLT_KEYSRC Or DDBLT_WAIT

End Sub

Public Sub Flip()

    'Flip the attached surface (the backbuffer) to the screen
    Primary.Flip Nothing, DDFLIP_WAIT
    
End Sub

Public Sub Terminate()

Dim i As Integer

    'This routine must destroy all surfaces and restore display mode
    Set Primary = Nothing
    Set BackBuffer = Nothing
    For i = 0 To UBound(Ship)
        Set Ship(i) = Nothing
    Next
    Call dd.RestoreDisplayMode
    Call dd.SetCooperativeLevel(FrmDirectX.hWnd, DDSCL_NORMAL)

End Sub
